@@ -176,11 +176,14 @@ def _StartCountStride(elem, shape, dimensions=None, grp=None, datashape=None,\
         # set unlim to True if dimension is unlimited and put==True
         # (called from __setitem__)
         if put and (dimensions is not None and grp is not None) and len(dimensions):
-            dimname = dimensions[i]
-            # is this dimension unlimited?
-            # look in current group, and parents for dim.
-            dim = _find_dim(grp, dimname)
-            unlim = dim.isunlimited()
+            try:
+                dimname = dimensions[i]
+                # is this dimension unlimited?
+                # look in current group, and parents for dim.
+                dim = _find_dim(grp, dimname)
+                unlim = dim.isunlimited()
+            except IndexError: # more slices than dimensions (issue 371)
+                unlim = False
         else:
             unlim = False
         # an iterable (non-scalar) integer array.
@@ -203,7 +206,7 @@ def _StartCountStride(elem, shape, dimensions=None, grp=None, datashape=None,\
                 elen = max(ea.max()+1,elen)
             else:
                 if ea.max()+1 > elen:
-                    msg="integer index exceeds dimension size" 
+                    msg="integer index exceeds dimension size"
                     raise IndexError(msg)
             eb = np.zeros(elen,np.bool)
             eb[ea] = True
@@ -244,36 +247,20 @@ Boolean array must have the same shape as the data along this dimension."""
             hasEllipsis = True
         # Replace boolean array with slice object if possible.
         elif ea.dtype.kind == 'b':
-            el = e.tolist()
-            if any(el):
-                start = el.index(True)
-                el.reverse()
-                stop = len(el)-el.index(True)
-                step = False
-                if e[start:stop].all():
-                    step = 1
+            if ea.any():
+                indices = np.flatnonzero(ea)
+                start = indices[0]
+                stop = indices[-1] + 1
+                if len(indices) >= 2:
+                    step = indices[1] - indices[0]
                 else:
-                    n1 = start+1
-                    ee = e[n1]
-                    estart = e[start]
-                    while ee != estart:
-                        n1 = n1 + 1
-                        ee = e[n1]
-                    step = n1-start
-                    # check to make sure e[start:stop:step] are all True,
-                    # and other elements in e[start:stop] are all False.
-                    ii = range(start,stop,step)
-                    for i in range(start,stop):
-                        if i not in ii:
-                            if e[i]: step = False
-                        else:
-                            if not e[i]: step = False
-                if step: # it step False, can't convert to slice.
-                    newElem.append(slice(start,stop,step))
+                    step = None
+                if np.array_equal(indices, np.arange(start, stop, step)):
+                    newElem.append(slice(start, stop, step))
                 else:
-                    newElem.append(e)
+                    newElem.append(ea)
             else:
-                newElem.append(slice(0,0))
+                newElem.append(slice(0, 0))
         else:
             newElem.append(e)
     elem = newElem
